/*****************************************************************************

  POPular -- A POP3 server and proxy for large mail systems

  $Id: t_ringd.c,v 1.1 2002/05/25 12:54:11 sqrt Exp $

  http://www.remote.org/jochen/mail/popular/

******************************************************************************

  Copyright (C) 1999-2002  Jochen Topf <jochen@remote.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA

*****************************************************************************/

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "../src/popular.h"
#include "../src/ringd.h"

int debug;


/*****************************************************************************

  send_command()

*****************************************************************************/
void
send_command(char *buf)
{
  char socket_name[MAXBUF];
  int fd;
  int local_socket;

  snprintf(socket_name, sizeof(socket_name)-1, "%s/pproxy.sock.%lu.%d", RUN_DIR, time(NULL), getpid());
  local_socket = uds_socket(socket_name, RINGD_SOCK_UMASK);
  if (local_socket < 0) {
    /* XLOG-DOC:SOS:0181:sock_open_failed
     * Pproxy tried to open a unix domain socket to talk to the ringd and
     * this failed */
    xlog_printf(xlog_sos, 0x0181, "sock_open_failed errno=%d errmsg='%s'", errno, strerror(errno));
    return;
  }
  if (uds_connect(local_socket, RINGD_SOCK_FILE) < 0) {
    /* XLOG-DOC:SOS:0182:sock_connect_failed
     * Pproxy tried to connect to the unix domain socket of the ringd and
     * this failed */
    xlog_printf(xlog_sos, 0x0182, "sock_connect_failed errno=%d errmsg='%s'", errno, strerror(errno));
    return;
  }

  if (send(local_socket, buf, strlen(buf)+1, 0) != strlen(buf)+1) {
    /* XLOG-DOC:SOS:0085:port_open_send_failed
     * Sending a port open request to the binder process failed. */
    xlog_printf(xlog_sos, 0x0085, "port_open_send_failed errno=%d errmsg='%s'", errno, strerror(errno));
    return;
  }

  {
    char buf[MAXBUF];
    int result = uds_recv(local_socket, NULL, buf, sizeof(buf), &fd);
    close(local_socket);
    unlink(socket_name);
    if (result < 0) {
      /* XLOG-DOC:SOS:0086:port_open_failed
       * Opening of the port failed for some reason on the pproxy side. */
      xlog_printf(xlog_sos, 0x0086, "port_open_failed errno=%d errmsg='%s'", errno, strerror(errno));
      return;
    } else if (result == 0) {
      /* XLOG-DOC:SOS:0143:port_open_no_fd
       * The opening and binding of the port by the binder process failed
       * for some reason. The message from the binder process is printed
       * with this message. */
      xlog_printf(xlog_sos, 0x0143, "port_open_no_fd msg='%s'", buf);
      return;
    }
  }
}


/*****************************************************************************

  main()

*****************************************************************************/
int
main(int argc, char *argv[])
{
  char buf[1024];
  int i;
  debug = DG_ALL;

  if (argc < 2) {
    fprintf(stderr, "argument is missing\n");
    exit(1);
  }

  buf[0] = '\0';
  for (i=1; i < argc; i++) {
    strlcat(buf, argv[i], sizeof(buf));
    strlcat(buf, " ", sizeof(buf));
  }
  if (buf[0] != '\0') {
    buf[strlen(buf)-1] = '\0';
  }

  fprintf(stderr, "sending command '%s'\n", buf);
  send_command(buf);

  return 0;
}


/** THE END *****************************************************************/
